/* OK1DXD (c) 2017
// Arduino PA control board test & calibration
// voltage divider sensors
// DS18B20 temperature sensor
// ACS712 current sensor
// FWD/REV sensors
// PTT line relay
// HighSWR fast comparator
// Min-Max values set & check
// 20x4 I2C LCD display
*/

#include <Wire.h>
#include <OneWire.h>
#include <DallasTemperature.h>
#include <LiquidCrystal_I2C.h> //modified I2C library !!!
#include <LcdBarGraphX.h>      //modified I2C library !!!

// Value definition
#define sensorPin 4   // digital pin for Dalas sensors
float temp1 = 0;      // to store temperature inside case
float temp2 = 0;      // to store temperature of PA transistor
int current = 0;      // to store current
float uref = 5.02 ;   // Voltage on Arduino reference pin
float voltage = 0;

// Max-Min values
float vmax = 33;   // if Voltage [Volts] > Vmax then Error
float vmin = 25;   // if Voltage [Volts] < Vmin then Error
float imax = 13;   // if Current [A] > imax then Error
float t1max = 45;  // if Temperature1 > max then Error
float t2max = 55;  // if Temperature2 > max then Error

// Calibration = (actual maesured/sensor value)* voltage divider ratio,
// need to be precise set here :
float divider = (29.1 / 29.2) * (15 + 2.2) / 2.2; // Ucc divider ratio
//float divider2 = (12 / 11.8) * (15 + 2.2) / 2.2;  // REV/FWD divider div ratio: 13
int i = 0 ; // loop counter

LiquidCrystal_I2C lcd(0x3F, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);
//   config(lcd_Addr, EN, RW, RS, D4, D5, D6, D7);   setBacklightPin(backlighPin, pol);
// sign for degree of Celsius
byte degree[8] = {B11100,
                  B10100,
                  B11100,
                  B00000,
                  B00000,
                  B00000,
                  B00000,
                  B00000
                 };
                 
//Dallas DS18B20 setup
OneWire oneWire(sensorPin); //digital pin for DS18B20
DallasTemperature sensors(&oneWire);
DeviceAddress tempSensor01 = {0x28, 0xFF, 0x4D, 0x33, 0x82, 0x15, 0x01, 0xED}; //ID comes here
DeviceAddress tempSensor02 = {0x28, 0xFF, 0x63, 0x7B, 0x82, 0x15, 0x01, 0x4B}; //ID comes here

// ACS712 sesitivity definition  mV 185 for 5A, 100 for 20A, 66 for 30A Sensor
//int mVperAmp = 185; 
int mVperAmp = 100;   
float acsoffset = uref / 2; // ofset for 0 A
double amps = 0;

//Digital IO pins setup
byte OnBoardLED = 13; // LED Arduino board
byte hiSWRpin = 2; // REV power detector on directional coupler
byte rele1 = 05;   // pin rele 1
byte  beep = 06;   // pin Piezo beep (must be a PWM pin)
byte  key1 = 8;   // pin tlacitko 1 Panel
byte  key2 = 03;   // pin tlacitko 2 PTT
byte digitalTEMP = 04;   // Dallas temp sensor

//Analog IO pins setup
// A4 - SDA SCL I2C pin
// A5 - SCL I2C pin
byte fwd = 6;   // pin Forward Power
byte rev = 7;   // pin Reflected Power
byte u24 = 3;   // pin Napajeci napeti /delic 1:11
byte current1 = 1; // pin mereni proudu

volatile byte swr_state = 0; //status of SWR 0 = OK, 1=ERROR

byte lcdNumCols = 16; //number of graph bar columns
LcdBarGraphX lbg1(&lcd, lcdNumCols, 4, 1); // Current
LcdBarGraphX lbg2(&lcd, lcdNumCols, 4, 2); // FWD
LcdBarGraphX lbg3(&lcd, lcdNumCols, 4, 3); // REV

void setup() {
 // Serial.begin(9600); //only when serial monitor dump needed
  pinMode(current1, INPUT);
  pinMode(rele1, OUTPUT);
  digitalWrite(rele1, LOW); // interrupt PTT line first
  pinMode(u24, INPUT);
  pinMode(key1, INPUT_PULLUP);// set pin to input
  pinMode(hiSWRpin, INPUT_PULLUP);// set interrupt pin to input
  digitalWrite(key1, HIGH);   // turn on pullup resistors

// Interrupt setup: the interrupt pin is falling to LOW when SWR ERROR
 attachInterrupt(digitalPinToInterrupt(hiSWRpin), hiSWRint, FALLING );
 
  // Start up the DallasTemperature library
  sensors.begin();
  // Set Measurement Resolution
  sensors.setResolution(tempSensor01, 9); // 9 bit is good enough
  sensors.setResolution(tempSensor02, 9); // 9 bit is good enough
  
  // -- initializing the LCD
  tone(beep, 2000, 200);
  lcd.begin(20, 4); //columns, rows
  lcd.clear();
  lcd.backlight();
  lcd.createChar(7, degree); // sign of degree

  // welcome screen & initial sensor test
  lcd.setCursor(0, 0);
  lcd.print("********************");
  lcd.setCursor(0, 1);
  lcd.print("*  OK1DXD (c)2017  *");
  lcd.setCursor(0, 2);
  lcd.print("* Arduino PA V 3.2 *");
  lcd.setCursor(0, 3);
  lcd.print("********************");
  delay (1000);

  lcd.clear();

  // Voltage test
  voltage = divider * (analogRead(u24)) * uref / 1023;
  lcd.setCursor(0, 0);
  lcd.print(voltage,1);
   lcd.setCursor(5, 0);
  if ( (voltage > vmin ) && (voltage < vmax ))
    lcd.print("Voltage.....OK");
  else
    lcd.print("Voltage..Fails");
  delay (300);

  // ACS712 test
  lcd.setCursor(0, 1);
  amps = ( (analogRead(current1)) * uref / 1023); //read the ADC and store the result in value
  amps = (amps * 1000 - acsoffset * 1000) / mVperAmp;
  amps = abs(amps);
  lcd.print(amps,1);
  lcd.setCursor(5, 1);
  if ( amps < 0.2 ) 
    lcd.print("ACS712......OK");
  else
    lcd.print("ACS712...Fails"); // if some current flows, or sensor is not connected
  delay (300);

  // DS18B20 test
  sensors.requestTemperatures(); // Send the command to get temperatures
  temp1 = sensors.getTempC(tempSensor02);
  lcd.setCursor(0, 2);
  lcd.print(temp1,0);
  lcd.setCursor(5, 2);
  if (temp1 > 0 )
    lcd.print("DS18B20.....OK");
  else
    lcd.print("DS18B20..Fails");

  pressKey(); // wait till a key is pressed
 
   tone(beep, 2000, 200); // GO!
   lcd.clear();
  
// if Uin and Current is on range, then PTT line released
   digitalWrite(rele1, HIGH); 
}

void loop()
{
 // test on HiSWR status
   if (swr_state == 1) {
    lcd.clear();
    lcd.setCursor(0, 1);
    lcd.print("*! HIGH SWR ERROR !*");
    pressKey(); // wait till a key is pressed
    swr_state = 0 ;// reset SWR ERROR status
    digitalWrite(rele1, HIGH); // connect PTT line when OK
    lcd.clear();
    }
  
  // reads analog Uin pin
  voltage = (analogRead(u24)) * uref / 1023;
  lcd.setCursor(0, 0);
  lcd.print("U=");
  lcd.print("    "); // clear space
  if (voltage >= 10)   lcd.setCursor(1, 0); // move decimal dot on right place
  else  lcd.setCursor(2, 0);
  lcd.print(divider * voltage, 1);

// Test on Under/Over voltage

   if ((divider * voltage )> vmax) {
    digitalWrite(rele1, LOW); // disconnect PTT line when ERROR
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print(divider * voltage);
    lcd.setCursor(0, 1);
    lcd.print("*OVER VOLTAGE ERROR*");
    pressKey(); // wait till a key is pressed
    digitalWrite(rele1, HIGH); // connect PTT line when OK
    lcd.clear();
     }

  // reads current sensor
  amps = 0;
  for (i = 0; i < 5; i++)  {
    amps = (amps + (analogRead(current1)) * uref / 1023); //read the ADC 5 times and store the result in value
    delay (25); }
  amps = (amps / 5); // get an AVERAGE absolute value
  amps = (amps * 1000 - acsoffset * 1000) / mVperAmp;
  amps = abs(amps);

   // Test on Over current
    if (amps > imax) {
    digitalWrite(rele1, LOW); // disconnect PTT line when ERROR
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print(amps);
    lcd.setCursor(0, 1);
    lcd.print("*OVER CURRENT ERROR*");
    pressKey(); // wait till a key is pressed
    digitalWrite(rele1, HIGH); // connect PTT line when OK
    lcd.clear();
     }

  lcd.setCursor(8, 0);
  lcd.print("     "); // clear spaces
  lcd.setCursor(7, 0);
  lcd.print("I=");
 // if (amps >= 10) lcd.setCursor(9, 0); // move decimal dot on right place
 //  else  lcd.setCursor(9, 0); // move one position left
  lcd.print(amps, 1);   // absolute value to avoid a minus sign or reverted sensor
  current = amps * 10;
  lcd.setCursor(0, 1);
  lcd.print("I1=:");
  lbg1.drawValue(amps, imax);

 
  // reads Dalas temperature sensors
  sensors.requestTemperatures(); // Send the command to get temperatures
  temp1= sensors.getTempC(tempSensor01); //inside the case
  lcd.setCursor(14, 0);
  lcd.print(temp1, 0);
  lcd.setCursor(16, 0);
  lcd.write(7);

   // Test on Over temperature
  temp2 = sensors.getTempC(tempSensor02); // PA module
  lcd.setCursor(17, 0);
  lcd.print(temp2, 0);
  lcd.setCursor(19, 0);
  lcd.write(7);

   // Test on Over temperature Case
    if (temp1 > t1max) {
    digitalWrite(rele1, LOW); // disconnect PTT line when ERROR
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print(temp1);
    lcd.setCursor(0, 1);
    lcd.print("* OVER TEMP1 ERROR*");
    pressKey(); // wait till a key is pressed
    digitalWrite(rele1, HIGH); // connect PTT line when OK
    lcd.clear();
     }
    // Test on Over temperature PA module    
    if (temp2 > t2max) {
    digitalWrite(rele1, LOW); // disconnect PTT line when ERROR
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print(temp2);
    lcd.setCursor(0, 1);
    lcd.print("* OVER TEMP2 ERROR*");
    pressKey(); // wait till a key is pressed
    digitalWrite(rele1, HIGH); // connect PTT line when OK
    lcd.clear();
     }
   
  // FWD bargraph
  lcd.setCursor(0, 2);
  lcd.print("FWD:");
  
  lbg2.drawValue(analogRead(fwd), 500); // full scale depends on coupler design  

  // REV bargraph
  lcd.setCursor(0, 3);
  lcd.print("REV:");
  lbg3.drawValue(analogRead(rev), 500); // full scale depends on coupler design

  delay(100);
}

// loop wait till a key is pressed
void pressKey() {
   Serial.println("! press KEY !");
   while (digitalRead(key1) == HIGH) {
   lcd.setCursor(0, 3);
   lcd.print("If READY Press <Key>"); 
   delay (500);
   tone(beep, 3000, 100);
   lcd.setCursor(0, 3);
   lcd.print("If READY Press      "); // wait till any key is pressed
   delay (500);
  }
  }
  
  // Interrupt routine when HiSWR ERROR - goes here
void hiSWRint() {
   digitalWrite(rele1, LOW); // interrupt PTT line first
   swr_state = 1; // SWR HIGH !!!
   delay (500);
 
}
